/*
*   nmgl.cpp
*/

#include <windows.h>
#include <GL/gl.h>
#include <stdio.h>
#include "nmTexture.h"
#include "ss_file.h"
#include "ss_haikei.h"
#include "ss_card.h"
#include "ss_atari.h"
#include "ss_tokuten.h"
#include "ss_syouhai.h"
#include "ss_com.h"
#include "ss_sound.h"
#include "ss_seffect.h"

#define NMGL_ITSELF

#include "nmgl.h"

#define STATE_PLAYER   0
#define STATE_COMPUTER 1
#define STATE_SYOUHAI  2

static int g_screenwidth, g_screenheight;

static int g_state = 0;

static int g_col;
static int g_row;
static int g_level;

static int player_point;
static int computer_point;

static int card1_index;
static int card2_index;

void nmgl_init(int w, int h){
  g_screenwidth = w;
  g_screenheight = h;

  player_point = 0;
  computer_point = 0;

  read_setting(&g_col, &g_row, &g_level);

  initTokuten();
  initSyouhai();
  initCard(g_col * g_row);
  initTexture();
  initCom(g_level);

  change_v(w, h, g_col, g_row);

  initSound("nms\\bgm.nms");
  initSEffect();
  readSEffect("nms\\open.nmse");

  playSound();

  // OpenGL ̐ݒ
  glDisable(GL_DEPTH_TEST);
  glCullFace(GL_BACK);
}
void nmgl_deinit(){
  deinitSEffect();
  deinitSound();
  deinitTexture();
  deinitCard();
  deinitSyouhai();
  deinitTokuten();
}

/* TCY̏ */
void nmgl_resize(int w, int h){
  g_screenwidth = w;
  g_screenheight = h;
  
  change_v(w, h, g_col, g_row);
  
  glViewport(0, 0, w, h);

  glMatrixMode(GL_PROJECTION);

  glLoadIdentity();

  // ˉe̎` AEAAAOA
  glFrustum(-1.4, 1.4, -1.0, 1.0, 2.0, 6.0);
  
  glMatrixMode(GL_MODELVIEW);
}

/* `̏ */
void nmgl_paint(void){
  int i, card_num;

  glClearColor(0.0f, 0.4f, 1.0f, 0.0f);
  glClear(GL_COLOR_BUFFER_BIT);

  paintHaikei();

  paintTokuten(player_point, computer_point);

  if(g_state != STATE_SYOUHAI){
    for(i = 0; i < (g_col * g_row); i++){
      glLoadMatrixf(getV(i));

      card_num = getCardNum(i);

      paintCard(i, card_num);
    }
  }
  else if(g_state == STATE_SYOUHAI){
    int syouhai;

    if(player_point - computer_point > 0){
      syouhai = 1;
    }
    else if(player_point - computer_point == 0){
	  syouhai = 2;
	}
    else if(player_point - computer_point < 0){
      syouhai = 3;
    }

    paintSyouhai(syouhai);
  }

  glFlush();
}

int nmgl_loop(){
  static int flag = 0;
  static int count = 10;

  loopHaikei();
  loopCard();

  // Rs[^[̏
  if(g_state == STATE_COMPUTER){
    int card1_num, card2_num;
    // flag: 0 vC[̃J[h𒲂ׂďԏ
    // flag: 4 Rs[^[̃J[h𒲂ׂďԏ
    if(flag == 0 || flag == 4){
      count--;
      if(count < 0){
        card1_num = getCardNum(card1_index);
        card2_num = getCardNum(card2_index);
        if(card1_num - card2_num == 0){
          // J[hv
          takeCard(card1_index);
          takeCard(card2_index);
          deleteComMemory(getCardNum(card1_index));
          if(flag == 0){
            count = 10;
            player_point++;
            g_state = STATE_PLAYER;
          }
          if(flag == 4){
            flag = 2;
            count = 5;
            computer_point++;
          }
        }
        else{
          // J[hsv
          closeCard(card1_index);
          closeCard(card2_index);
          if(flag == 0){
            flag = 2;
            count = 5;
          }
          if(flag == 4){
            flag = 0;
            count = 10;
            g_state = STATE_PLAYER;
          }
        }
      }
    }
    // Rs[^[̎vl Pڂ̃J[h߂
    else if(flag == 2){
      count--;
      if(count < 0){
        // L̒ɂQ̈vJ[h邩H
        if(getComFirstThink(&card1_index, &card2_index)){
          openCard(card1_index);
          addComMemory(getCardNum(card1_index), card1_index);
        }
        else{
          // _łPڂI
          card1_index = getRandomCard();
          openCard(card1_index);
          addComMemory(getCardNum(card1_index), card1_index);
          // JꂽPƋL̒̂P̈vJ[h邩H
          if(!getComSecondThink(getCardNum(card1_index), card1_index, &card2_index)){
            // _łQڂI
            card2_index = getRandomCard();
          }
        }
        playSEffect();
        count = 10;
        flag = 3;
      }
    }
    // Qڂ̃J[h߂
    else if(flag == 3){
      count--;
      if(count < 0){
		playSEffect();
        openCard(card2_index);
        addComMemory(getCardNum(card2_index), card2_index);
        count = 10;
        flag = 4;
      }
    }
	// s̔
	if((player_point + computer_point) - ((g_col * g_row) / 2) >= 0){
	  g_state = STATE_SYOUHAI;
      count = 50;  
	}
  }
  else if(g_state == STATE_SYOUHAI){
    count--;
    if(count < 0){
      flag = 0;
      count = 10;
      player_point = 0;
      computer_point = 0;
      g_state = STATE_PLAYER;
      initCard(g_col * g_row);
      initCom(g_level);
    }
  }

  return 1;
}

// }EXNbN̑΂铮
void nmgl_mousedown(int x, int y){
  int index;
  static int flag = 0;

  // vC[̏
  if(g_state == STATE_PLAYER){
    index = atariCard(x, y, g_col * g_row);

    if(getCardState(index) != STATE_CLOSE)return;

    if(index >= 0){
      openCard(index);
      addComMemory(getCardNum(index), index);
      // Pڂ̃J[h߂
      if(flag == 0){
        playSEffect();
        card1_index = index;
        flag = 1;
      }
      // Qڂ̃J[h߂
      else{
        playSEffect();
        card2_index = index;
        flag = 0;
        g_state = STATE_COMPUTER;
      }
    }
  }
}
