#include <windows.h>
#include "nmgl.h"

#define SCREEN_FPS 60

#define SCREEN_WIDTH 420
#define SCREEN_HEIGHT 300

#define WINDOW_CLASS_NAME "nmss3d"
#define WINDOW_NAME "nm_o3d"

/* ϐ */
static HWND g_hWnd;
static int g_bActive = 0;
static DWORD waittime;

/* ֐ */
BOOL bSetupPixelFormat(HDC hdc);
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR, int nCmdShow)
{
  MSG msg;
  WNDCLASS wc;
  HWND hWnd;

  // Register Class
  if(hPrevInst == NULL)
  {
    ZeroMemory(&wc, sizeof(wc));
    wc.lpfnWndProc = (WNDPROC)WndProc;
    wc.hInstance = hInst;
    wc.hIcon = LoadIcon(hInst, "MYICON");
    wc.hCursor = LoadCursor((HINSTANCE)NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)COLOR_BACKGROUND;
    wc.lpszClassName = WINDOW_CLASS_NAME;

    if(RegisterClass(&wc) == 0){
      return 0;
    }
  }
  
  // CreateWindow
  hWnd = CreateWindow(WINDOW_CLASS_NAME, WINDOW_NAME, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, (HWND)NULL, (HMENU)NULL, hInst, (LPVOID)NULL);

  if(hWnd == NULL){
    return 0;
  }

  g_hWnd = hWnd;

  nmgl_init(SCREEN_WIDTH, SCREEN_HEIGHT);

  ShowWindow(hWnd, nCmdShow);
  UpdateWindow(hWnd);

  msg.wParam = 0;

  while(1){
    waittime = GetTickCount();

    if(g_bActive){
      nmgl_loop();
      InvalidateRect(hWnd, NULL, FALSE);
    }
    else{
      WaitMessage();
    }
    do{
      if(PeekMessage(&msg, (HWND)NULL, 0, 0, PM_REMOVE)){
        if(msg.message == WM_QUIT)goto fin;
        TranslateMessage(&msg);
        DispatchMessage(&msg);
      }
    }while(GetTickCount() < waittime + SCREEN_FPS);
  }

fin:

  nmgl_deinit();

  return msg.wParam;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam){
  static HDC hdc;
  static HGLRC hrc;

  switch(uMsg){
    case WM_CREATE:
      RECT wr, cr;
      GetWindowRect(hWnd, &wr);
      GetClientRect(hWnd, &cr);
      MoveWindow(hWnd, wr.left, wr.top, (wr.right - wr.left) + SCREEN_WIDTH - (cr.right - cr.left), (wr.bottom - wr.top) + SCREEN_HEIGHT - (cr.bottom - cr.top), TRUE);
      // OpenGL̐ݒ
      hdc = GetDC(hWnd);
      bSetupPixelFormat(hdc);
      hrc = wglCreateContext(hdc);
      wglMakeCurrent(hdc, hrc);
      return 0;
    case WM_SIZE:
      RECT nr;
      GetClientRect(hWnd, &nr);
      nmgl_resize(nr.right, nr.bottom);
      return 0;
    case WM_PAINT:
      nmgl_paint();
      SwapBuffers(hdc);
      return 0;
    case WM_ACTIVATE:
      g_bActive = HIWORD(wParam)?0:1;
      return 0;
    case WM_LBUTTONDOWN:
      nmgl_mousedown(LOWORD(lParam), HIWORD(lParam));
      return 0;
    case WM_CLOSE:
      wglMakeCurrent(hdc, 0);
      wglDeleteContext(hrc);
      ReleaseDC(hWnd, hdc);
      DestroyWindow(hWnd);
      return 0;
    case WM_DESTROY:
      PostQuitMessage(0);
      return 0;
  }

  return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

/* sNZtH[}bg̐ݒ */
BOOL bSetupPixelFormat(HDC hdc)
{
  static PIXELFORMATDESCRIPTOR pfd = {
    sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd/
    1,                              // version number
    PFD_DRAW_TO_WINDOW |            // support window
    PFD_SUPPORT_OPENGL |            // support OpenGL
    PFD_DOUBLEBUFFER,               // double buffered
    PFD_TYPE_RGBA,                  // RGBA type
    24,                             // 24-bit color depth
    0, 0, 0, 0, 0, 0,               // color bits ignored
    0,                              // no alpha buffer
    0,                              // shift bit ignored
    0,                              // no accumulation buffer
    0, 0, 0, 0,                     // accum bits ignored
    0,                              // no z-buffer
    0,                              // no stencil buffer
    0,                              // no auxiliary buffer
    PFD_MAIN_PLANE,                 // main layer
    0,                              // reserved
    0, 0, 0                         // layer masks ignored
  };

  int pixelformat;

  if((pixelformat = ChoosePixelFormat(hdc, &pfd)) == 0){
    MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK);
    return FALSE;
  }

  if(SetPixelFormat(hdc, pixelformat, &pfd) == FALSE)    {
    MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK);
    return FALSE;
  }

  return TRUE;
}
  
